#include <stdlib.h>
#include <string.h>
#include "debug.h"
#include "array.h"
#include "debug.h"

/*
** Cree un tableau vide
**************************
*/

array_t* createEmptyArray (int size)
{
  array_t* array;

  debugIf(size<0,"invalid array index");

  array      = (array_t*) malloc (sizeof(array_t));
  array->dot = (dot_t*)   malloc (sizeof(dot_t)*size);

  array->nber = size;
  memset(array->dot,0,sizeof(dot_t)*size);

  return array;
}

/*
** Transform an array in array_t*
***********************************
*/

array_t* createArray (int size,dot_t* dot)
{
  int i;
  array_t* array;

  debugIf(size!=0 && !dot,"Can't copy data from null pointer");

  array = createEmptyArray(size);
  for(i=0; i<size; i++)
	setArrayDot(array,i,&dot[i]);

  return array;
}

/*
** Duplicate an array
***********************
*/

array_t* createDupArray (array_t* array)
{
  array_t* copy;
 
  debugIf(!array,"Coder doesn't want to dup null array");

  copy = createEmptyArray(getArraySize(array));
  copyArray(copy,array,0,getArraySize(array));
  return copy;
}



array_t* deleteArray (array_t* array)
{
  debugIf(!array,"New-York !! Lalala  ... keep our calm Ally ..");

  free(array->dot);
  free(array);
  array = NULL;
  return array;
}



/*
** Copy an array in another
******************************
*/


array_t* copyArray (array_t* dst, array_t* src, int position, int number)
{
  int i;

  debugIf(position+number>dst->nber,"copying out of array");

  for(i=0;i<number;i++)
	setArrayDot(dst,position+i,&(src->dot[i]));

  return dst;
}


/*
** get a dot from an array
****************************
*/

dot_t* getArrayDot (array_t* array, int pos)
{
  debugIf(!array,"array is null");
  debugIf(pos<0,"invalid array index");
  debugIf(pos>=getArraySize(array),"Invalid array index");
  return &(array->dot[pos]);
}


/*
** set a dot from an array
****************************
*/

dot_t* setArrayDot (array_t* array, int pos, dot_t* dot)
{
  debugIf(!array,"array is null");
  debugIf(pos<0,"invalid array index");
  debugIf(pos>=getArraySize(array),"Invalid array index");

  copyDot(&(array->dot[pos]),dot);
  return dot;
}

/*
** No comment
********************
*/

int getArraySize (array_t* array)
{
  debugIf(!array,"array is null");
  return array->nber;
}


/*
** Update the array maximum distance between two dots
*******************************************************
*/

double calcArrayMax (array_t* array)
{
  int    pos;
  double calc;
  double max;

  debugIf(!array,"array is null");

  max = 0.0;  
  
  for(pos=0; pos< getArraySize(array)-2; pos++)
	{
	  debugIf(!&(array->dot[pos]),"You must set previous dot before");  
	  calc = distance(&(array->dot[pos]),&(array->dot[pos+1]));
	  if(calc > max) max = calc;
	}

  return max;
}

/*
** Debug affichage
*******************
*/

void printArray (array_t* array)
{
  int index;

  debugIf(!array, "the array to print is null");

  for(index=0; index<getArraySize(array); index++)
    {
	  printf("%3d  ",index);	 
	  printDot(getArrayDot(array,index));
	  printf("\n");
    }
}
