#include "bezier.h"
#include "io.h"
#include "debug.h"


/*
** deCasteljau 
***************
*/

array_t* deCasteljau (array_t* array)
{
  register int i;
  array_t* middle;
  array_t* back;
  array_t* calc;
  
  debugIf(!array, "array_t pointer is null");
  
  if(getArraySize(array) > 1)
    {
      middle = createEmptyArray(getArraySize(array)-1);
      
      for(i=0;i<getArraySize(middle);i++)
	setArrayDot(middle,i,calcul_middle(getArrayDot(array,i),getArrayDot(array,i+1)));
      
      calc = deCasteljau(middle);
      back = createEmptyArray(getArraySize(calc)+2);
      
      setArrayDot(back,0,                   getArrayDot(array,0));
      setArrayDot(back,getArraySize(back)-1,getArrayDot(array,getArraySize(array)-1));
      copyArray(back,calc,1,getArraySize(calc));
      
      deleteArray(middle);
      deleteArray(calc);
    }
  else
    back = createDupArray(array);
  
  return back;
}

/*
** deCasteljau Recursif avec condition d arret
***********************************************
*/

array_t* optimalDeCasteljau (array_t* array, double distance, const char* name)
{
  array_t* before;
  array_t* after;

  debugIf(distance >  1.0, "distance range ]0.0,1.0]");
  debugIf(distance <= 0.0, "distance rance ]0.0,1.0]");
  
  if(calcArrayMax(array) > distance)
    {
      writePs(name,array);
      before = deCasteljau(array);	  
      after  = optimalDeCasteljau (before,distance,name);
      deleteArray(before);
    }
  else
    {
      after = createDupArray(array);
      writePs(name,after);
    }
  
  return after;
}

