#ifndef INCLUDE_DEBUG
#define INCLUDE_DEBUG

#include <stdio.h>

/*
  __LINE__ can be a string on other compilers
  __FUNCTION__ like __PRETTY_FUNCTION__ is gnu specific
*/


#ifndef NDEBUG

/* Private Functions */

#  define printLocation()											\
   printf															\
   (																\
     "----------------------------------------------\n"				\
     "-- Compiled the " ## __DATE__ ## " at " ## __TIME__ ## "\n"	\
     "-- Function " ## __FUNCTION__ ## "\n"							\
     "-- File     %s\n"												\
     "-- Line     %d \n"											\
     "----------------------------------------------\n",			\
     __FILE__, __LINE__												\
   );																\
   fflush(stdout);

/* Public Functions */

#  define debugFatal(text)									\
   do														\
   {														\
     printf													\
     (														\
       "\n"													\
       "----------------------------------------------\n"	\
       "-- Reason : %s\n",									\
       text													\
     );														\
     printLocation();										\
     callDebugger();										\
     /* Include asm int 0x03 here to call the debugger */	\
   } while (0)

#  define debugIf(test, text)								\
   do														\
   {														\
     if(test)												\
     {														\
	   printf												\
       (													\
         "\n"												\
         "----------------------------------------------\n"	\
         "-- %s\n"											\
         "-- Test done : " ## #test ## "\n",				\
         text												\
       );													\
	   printLocation();										\
       callDebugger();										\
     }														\
   } while (0)

#  ifdef __i386__
#    define callDebugger() __asm__ __volatile__ ("int $0x03\n")
#  else
#    define callDebugger() /* linux int3() instruction in asm */ debugFatal ("callDebugger Only works on Intel architecture")
#  endif

#else

#  define debugFatal(text)   /* No error report indicating text          */ do {} while (0)
#  define debugIf(test,text) /* No error check with test outputting text */ do {} while (0)
#  define callDebugger()     /* linux int3() instruction in asm          */ do {} while (0)

#endif

#endif

