#include <stdlib.h>
#include <math.h>

#include "debug.h"
#include "dot.h"


/*
** Variables globales
**********************
*/

int Dimension;


/*
** Fixe/Lit la dimension des points (x,y,z, etc ...)
*****************************************************
*/


void setDimension (int dimension)
{
  debugIf(dimension<2,"dimension quite small, no ?");

  Dimension = dimension;
}

int getDimension (void)
{
  return Dimension;
}


/*
** Affecte a un point une valeur a une de ses dimensions
**********************************************************
*/


dot_t* setDotData (dot_t* dot, int dimension, double value)
{
  debugIf(dimension>=getDimension(), "dimension is invalid");
  
  (*dot)[dimension] = value;
  return dot;
}

/*
** Recupere la valeur d une des dimension d un point
*****************************************************
*/

double getDotData (dot_t* dot, int dimension)
{
  debugIf(!dot, "can t get data from an null dot");
  debugIf(dimension>=getDimension(), "dimension is invalid");

  return (*dot)[dimension];
}

/*
** Fait une copie d un point
*******************************************
*/

dot_t* copyDot(dot_t* dest, dot_t* src)
{
  int index;

  debugIf(!dest, "can t get data to an null dot");
  debugIf(!src,  "can t get data from an null dot");

  for(index=0; index<MAX_DIMENSION; index++)
	(*dest)[index] = (*src)[index];

  return dest;
}


/*
** Retourne la distance euclidienne entre deux points
******************************************************
*/

double distance (dot_t* one, dot_t* two)
{
  int dim;
  double square;
  double result;

  result = 0;

  debugIf(getDimension() !=2, "The algorithm is for dim 2 ..");

  for(dim=0; dim<getDimension(); dim++)
	{
	  square  = ((*one)[dim]-(*two)[dim]);
	  square *= square;
	  result += square;
	}

  return sqrt(result);
}


/*
** Calcule le milieu d un segment
***********************************
*/

dot_t* calcul_middle (dot_t* one, dot_t* two)
{
  int index;
  static dot_t result;
 
  for(index=0; index<getDimension(); index++)
	result[index] = ((*one)[index]+(*two)[index])/2;

  return &result;
}


/*
** Debug affichage 
*******************
*/

void printDot (dot_t* dot)
{  
  int dimension;

  debugIf(!dot, "the dot to print is null");

  for(dimension=0; dimension< getDimension(); dimension++)
	printf("%1.2f  ", getDotData(dot,dimension));
}

