#include <stdlib.h>
#include <string.h>
#include "espace.h"
#include "debug.h"
#include "io.h"

/*
** Create a new space
****************************
*/


espace_t* createEspace (int size_line, int nb_column)
{
  espace_t* espace;
  
  debugIf(size_line < 0, "invalid espace size");
  debugIf(nb_column < 0, "invalid espace size");
  
  espace      = (espace_t*) malloc (sizeof(espace_t));
  espace->dot = (dot_t*)    malloc (sizeof(dot_t)*size_line*nb_column);
  
  espace->size_line = size_line;
  espace->nb_column = nb_column;

  memset(espace->dot,0,sizeof(dot_t)*size_line*nb_column);

  return espace;
}

/*
** Create a new space as a part of a existing one
***************************************************
*/

espace_t* createSubEspace (espace_t* espace, int pos_x, int pos_y, int size_x, int size_y)
{
  int x,y;
  espace_t* new_espace;

  debugIf(size_x <=0, "Pas possible !!");
  debugIf(size_y <=0, "Pas possible !!");
  debugIf(pos_x+size_x > getEspaceLine(espace),   "Out of X espace access");
  debugIf(pos_y+size_y > getEspaceColumn(espace), "Out of Y espace access");

  new_espace = createEspace(size_x,size_y);

  for(y=0; y<size_y;y++)
	for(x=0; x<size_x;x++)
	  setEspaceDot(new_espace,x,y,getEspaceDot(espace,x+pos_x,y+pos_y));

  return new_espace;
}

/*
** Make an array from an espace line
*************************************
*/

array_t* createEspaceArrayFromLine (espace_t* espace ,int column)
{
  int i;
  array_t* array;

  debugIf(!espace,"Argh .. Onece agin a null pointer");
  debugIf(column >= getEspaceColumn(espace),"copying out of array");

  array = createEmptyArray(getEspaceLine(espace));

  for(i=0; i<getArraySize(array); i++)
	setArrayDot(array,i,getEspaceDot(espace,i,column));
 
  return array;
}

/*
** Make an array from an espace column
***************************************
*/

array_t* createEspaceArrayFromColumn (espace_t* espace, int line)
{
  int i;
  array_t* array;

  debugIf(!espace,"Dont thrown me by the window for a null pointer !!");
  debugIf(line >= getEspaceLine(espace),"copying out of array");

  array = createEmptyArray(getEspaceColumn(espace));

  for(i=0; i<getArraySize(array); i++)
	setArrayDot(array,i,getEspaceDot(espace,line,i));
 
  return array;
}

/*
** Free all space resources
******************************
*/

espace_t* deleteEspace (espace_t* espace)
{
  debugIf(!espace,"Oups !! deleting null pointer !!!");

  free(espace->dot);
  free(espace);
  espace = NULL;
  return espace;
}

/*
** get a dot from an espace
**************************************
*/

dot_t* getEspaceDot (espace_t* espace, int x, int y)
{
  debugIf(!espace,"espace is null");
  debugIf(x<0,"invalid espace index");
  debugIf(y<0,"invalid espace index");
  debugIf(x >= getEspaceLine(espace),"Invalid array index");
  debugIf(y >= getEspaceColumn(espace),"Invalid array index");

  return &(espace->dot[y*getEspaceLine(espace)+x]);
}
/*
** set a dot of an espace
**************************************
*/

dot_t* setEspaceDot (espace_t* espace, int x, int y, dot_t* dot)
{
  debugIf(!espace,"espace is null");
  debugIf(x<0,"invalid espace index");
  debugIf(y<0,"invalid espace index");
  debugIf(x >= getEspaceLine(espace),"Invalid array index");
  debugIf(y >= getEspaceColumn(espace),"Invalid array index");

  copyDot(&(espace->dot[y*getEspaceLine(espace)+x]),dot);
  return dot;
}

/*
** Copy an array to an espace line
*************************************
*/

espace_t* copyLineToEspace (int column, espace_t* espace, array_t* array)
{
  int i;

  debugIf(!espace,"Classical !!! Null pointer");
  debugIf(!array, "Kick ass  !! null pointer");

  debugIf(column >= getEspaceColumn(espace),"copying out of array");
  debugIf(getArraySize(array) != getEspaceLine(espace),"... !!!! @@@@ ####");

  for(i=0; i<getArraySize(array); i++)
	setEspaceDot(espace,i,column,getArrayDot(array,i));

  return espace;
}

/*
** Copy an array to an espace column
**************************************
*/

espace_t* copyColumnToEspace (int line, espace_t* espace, array_t* array)
{
  int i;

  debugIf(!espace,"Guess what happened !! A null pointer");
  debugIf(!array,"Guess what happened !! Stupid me");
  debugIf(line >= getEspaceLine(espace),"copying out of array");
  debugIf(getArraySize(array) != getEspaceColumn(espace),"... !!!! @@@@ ####");

  for(i=0; i<getArraySize(array); i++)
	setEspaceDot(espace,line,i,getArrayDot(array,i));

  return espace;
}

/*
** Get Espace dimension
******************************
*/

int getEspaceLine (espace_t* espace)
{
  debugIf(!espace,"Surprise !! null pointer !!");
  return espace->size_line;
}


int getEspaceColumn (espace_t* espace)
{
  debugIf(!espace,"And the coder used a null pointer");
  return espace->nb_column;
}


/*
** Debug affichage de la liste 
********************************
*/

void printEspace (espace_t* espace)
{
  int x,y;

  debugIf(!espace,"Can't try to find a bug on a null pointer !!");

  for(y=0; y<getEspaceColumn(espace);y++)
	{
	  for(x=0; x<getEspaceLine(espace);x++)
		{
		  printf("  [%d %d]   ",x,y);	 
		  printDot(getEspaceDot(espace,x,y));
		}
	  printf("\n");
	}
}
