#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "debug.h"
#include "image.h"
#include "io.h"

/*
** Create a black image
**************************************
*/

image_t* createImage (int size_x, int size_y)
{
  image_t* image;

  image       = (image_t*) malloc (sizeof(image_t));
  image->data = (char*)    malloc (size_x*size_y*3);

  setImageSizeX(image,size_x);
  setImageSizeY(image,size_y);
  memset(image->data,0,size_x*size_y*3);

  return image;
}

/*
** Free an image ressource
*********************************
*/

void deleteImage (image_t* image)
{
  free(image->data);
  free(image);
}


/*
** Get an image size proprieties
*********************************
*/


int getImageSizeX (image_t* image)
{
  debugIf(!image,"null pointer");

  return image->size_x;
}

int getImageSizeY (image_t* image)
{
  debugIf(!image,"null pointer");

  return image->size_y;
}

/*
** Set an image size proprieties
*********************************
*/


image_t* setImageSizeX (image_t* image, int x)
{
  debugIf(!image,"null pointer");

  image->size_x = x;
  return image;
}

image_t* setImageSizeY (image_t* image, int y)
{
  debugIf(!image,"null pointer");

  image->size_y = y;
  return image;
}

/*
** Get direct buffer reference to an image pixel
*************************************************
*/


pixel_t getImagePixel (image_t* image ,int x, int y)
{
  debugIf(!image,"null pointer");

  return (pixel_t) &image->data[(y*getImageSizeX(image)+x)*3];
}

/*
** Set the color of a pixel of an image
*****************************************
*/

image_t* setImagePixel (image_t* image ,int x, int y, pixel_t pixel)
{
  debugIf(!image,"null pointer");

  image->data[(y*getImageSizeX(image)+x)*3+0] = pixel[0];
  image->data[(y*getImageSizeX(image)+x)*3+1] = pixel[1];
  image->data[(y*getImageSizeX(image)+x)*3+2] = pixel[2];

  return image;
}

/*
** Print an image information
******************************
*/

image_t* printImage(image_t* image)
{
  debugIf(!image,"null pointer");

  printf("Size X : %d\n", getImageSizeX(image));
  printf("Size Y : %d\n", getImageSizeY(image));

  return image;
}
