#include <stdlib.h>
#include "io.h"
#include "debug.h"

#define FALSE 0
#define TRUE  1

#define MAGNIFY 200
int START[2] = {100,300};


/*
** Ecrite un tableau de point a la suite d un fichier Poscript
****************************************************************
*/

void writePs (const char* name, array_t* array)
{  
  FILE*   file;
  int     index;
  int     dimension;

  debugIf(getArraySize(array)<1, "the list to print is null");
  debugIf(getDimension() != 2  , "the dimension must be two");

  file=fopen(name,"a");

  for(dimension=0; dimension< getDimension(); dimension++)
    fprintf(file, "%f ", getDotData(getArrayDot(array,0),dimension)*MAGNIFY+START[dimension]);
	    
  fprintf(file,"moveto\n");

  for(index=1; index<getArraySize(array); index++)
    {
      for(dimension=0; dimension< getDimension(); dimension++)
		fprintf(file, "%f ", getDotData(getArrayDot(array,index),dimension)*MAGNIFY+START[dimension]);

      fprintf(file,"lineto\n");
    }

  //fprintf(file,"%d %d scale\n",MAGNIFY,MAGNIFY);
  fprintf(file,"stroke\n");
  fprintf(file,"showpage\n");
  fclose(file);
}



/*
** Read non comment non void lines
***********************************
*/

char* readLine(FILE* file, char* line, int size)     
{
  int restart;

  debugIf(!file,"null pointer");

  do
	{
      if (fgets(line,size,file) == 0)
		return NULL;

	  if(*line == '#' || *line == '\n')
		{
		  if(line[strlen(line)-1] != '\n')
			while(fgetc(file) != '\n')
			  {}
		  restart = TRUE;
		}
	  else
		{
		  restart = FALSE;
		  if(line[strlen(line)-1] != '\n')
			line = NULL;
		}		 		   
    } while (restart);

  return line;
}

/*
** Create a image from a pnm file
**********************************
*/

image_t* readPnm (const char* name)   
{
  FILE*    file;
  image_t* image;

  int   i;
  int   sx, sy;
  char  type[5];
  char  size[21];
  char  number[6];

  debugIf(!name,"null pointer");

  file = fopen(name,"r+");

  if(!file)
	return NULL;

  if(!readLine(file,type,4))
	return NULL;

  if (*type != 'P' || *(type+1) != '6')  
	return NULL;

  if(!readLine(file,size,20))
	return NULL;

  sscanf(size,"%d%d",&sx,&sy); 

  if(!readLine(file,number,5))
	return NULL;

  if(strcmp(number,"255\n")!=0)
	return NULL;

  image = (image_t*) malloc (sizeof(image_t));
  
  if(image == NULL)
	return NULL;

  setImageSizeX(image,sx);
  setImageSizeY(image,sy);

  image->data = (char*) malloc (sx*sy*3);

  for (i=0; i<sx*sy*3; i++)
	image->data[i] = (char) fgetc (file);

  return image;
}

/*
** write a pnm file from an image
**********************************
*/


void writePnm(const char* name, image_t* image)
{
  FILE* file;
  int   i; 

  debugIf(!name,"null pointer");

  file = fopen(name,"w");

  fprintf(file, "P6\n");
  fprintf(file, "#File name : %s\n",name);
  fprintf(file, "%d %d\n",getImageSizeX(image),getImageSizeY(image));
  fprintf(file, "255\n");

  for (i=0; i<getImageSizeX(image)*getImageSizeY(image)*3; i++)
	fputc(image->data[i],file);

  fclose(file); 
}

/*
** Make a espace from a espace file
*************************************
*/

espace_t* readEspace (const char* name)
{
  int   i,x,y;
  FILE* file;

  char  line [51];
  char  order[51];

  float value;
  int   dimension;
  int   degree_x;
  int   degree_y;
  float divizer;

  dot_t     dot;
  espace_t* espace;

  dimension = -1;
  degree_x  = -1;
  degree_y  = -1;
  divizer   = -1;

  file = fopen (name,"r");

  if(!file)
	return NULL;

  for(i=0; i<4; i++)
	{
	  readLine(file,line,50);

	  if(!line)
		return NULL;

	  sscanf(line,"%s %f",order,&value);

	  if (strcmp(order,"Dimension") == 0)
		dimension = value;
	  else if (strcmp(order,"DegreeX") == 0)
		degree_x = value;
	  else if (strcmp(order,"DegreeY") == 0)
		degree_y = value;
	  else if (strcmp(order,"Divizer") == 0)
		divizer = value;
	  else
		return NULL;
	}

  if (dimension == -1 || degree_x  == -1 || degree_y  == -1 || divizer == -1)
	return NULL;

  setDimension(dimension);
  espace = createEspace(degree_x,degree_y);

  if(!espace)
	{
	  free(espace);
	  return NULL;
	}

  for(y=0; y< degree_y; y++)
	{
	  for(x=0; x< degree_x; x++)
		{
		  fscanf(file,"[ ");
		  for(i=0; i<dimension ; i++)
			{			  
			  fscanf(file,"%f",&value);
			  setDotData(&dot,i,value/divizer);
			}
		  fscanf(file," ] ");
		  setEspaceDot(espace,x,y,&dot);
		}
	  fscanf(file,"\n");
	}

  return espace;
}
