#include <stdlib.h>
#include "debug.h"
#include "bezier.h"
#include "morphing.h"
#include "box.h"


image_t* ImageResult;
image_t* ImageSource;

void setMorphingSource (image_t* image)
{
  ImageSource = image;
}

void setMorphingResult (image_t* image)
{
  ImageResult = image;
}

void
morphing (espace_t* espace,	box_t     box )
{
  register int i;

  int size_x   , size_y;
  int half_x   , half_y;
  int middle_x , middle_y;
  
  array_t*  array_tmp;
  array_t** array;
  espace_t* new_espace;

  debugIf(!ImageSource, "Image Source not defined");
  debugIf(!ImageResult, "Image Source not defined");
  debugIf(getDimension() != 2 ,"morphing on works for dimension 2\n");

  /* It would be possible to optimize here stopping as soon as 
     box.size.x/y <= getEspaceLine/Column
  */

  if(box.size.x == 1 && box.size.y == 1)
	{
	  static int progress = 0;
	  dot_t* dot;

	  progress = (progress+1)%(getImageSizeX(ImageSource)*getImageSizeY(ImageSource));
	  printf("\rProgression : %03.02f  %%",(float)progress*100/(getImageSizeX(ImageSource)*getImageSizeY(ImageSource)));

	  //dot = getEspaceDot(espace,0,0);
	  dot = getEspaceDot(espace,getEspaceLine(espace)/2,getEspaceColumn(espace)/2);
	  	  
	  setImagePixel
		(ImageResult,
		 box.pos.x, box.pos.y,
		 getImagePixel
		 (ImageSource,	  
		  getDotData(dot,0)*getImageSizeX(ImageSource),
		  getDotData(dot,1)*getImageSizeY(ImageSource)
		  )
		 );
	}
  
  size_x     = getEspaceLine(espace);
  size_y     = getEspaceColumn(espace);
  new_espace = espace;

  // Apply bezier transformation on y

  array = (array_t**) malloc (sizeof(array_t*)*size_x);

  for(i=0; i<size_x; i++)
	{
	  array_tmp = createEspaceArrayFromColumn(new_espace,i);
	  array[i]  = deCasteljau(array_tmp);
	  deleteArray(array_tmp);
	}

  size_y = getArraySize(array[0]);
  new_espace = createEspace(size_x,size_y);

  for(i=0; i<size_x; i++)
	{
	  copyColumnToEspace(i,new_espace,array[i]);
	  deleteArray(array[i]);
	}
  
  free(array);

  // Apply bezier transformation on X

  array = (array_t**) malloc (sizeof(array_t*)*size_y);

  for(i=0; i<size_y; i++)
	{
	  array_tmp = createEspaceArrayFromLine(new_espace,i);
	  array[i]  = deCasteljau(array_tmp);
	  deleteArray(array_tmp);
	}

  deleteEspace(new_espace);

  size_x = getArraySize(array[0]);

  new_espace = createEspace(size_x,size_y);

  for(i=0; i<size_y; i++)
	{
	  copyLineToEspace(i,new_espace,array[i]);
	  deleteArray(array[i]);
	}
  
  free(array);

  half_x   = getEspaceLine(espace);
  half_y   = getEspaceColumn(espace);

  middle_x = half_x - 1;
  middle_y = half_y - 1;
	
  if(box.size.x > 1 && box.size.y > 1)
	{
	  espace_t* espace_ul;
	  espace_t* espace_ur;
	  espace_t* espace_dl;
	  espace_t* espace_dr;

	  espace_ul = createSubEspace(new_espace, 0,        0,        half_x, half_y);
	  espace_ur = createSubEspace(new_espace, middle_x, 0,        half_x, half_y);
	  espace_dl = createSubEspace(new_espace, 0,        middle_y, half_x, half_y);
	  espace_dr = createSubEspace(new_espace, middle_x, middle_y, half_x, half_y);
	  
	  morphing (espace_ul, boxUL(box)); 
	  deleteEspace(espace_ul);
	  
	  morphing (espace_ur, boxUR(box));
	  deleteEspace(espace_ur);
	  
	  morphing (espace_dl, boxDL(box));
	  deleteEspace(espace_dl);
	  
	  morphing (espace_dr, boxDR(box));
	  deleteEspace(espace_dr);
	}
  else if(box.size.x > 1)
	{
	  espace_t* espace_l;
	  espace_t* espace_r;

	  espace_l = createSubEspace(new_espace, 0,        0,        half_x, 1);
	  espace_r = createSubEspace(new_espace, middle_x, 0,        half_x, 1);
	  
	  morphing (espace_l, boxL(box)); 
	  deleteEspace(espace_l);
	  
	  morphing (espace_r, boxR(box));
	  deleteEspace(espace_r);
	}
  else if(box.size.y > 1)
	{
	  espace_t* espace_u;
	  espace_t* espace_d;

	  espace_u = createSubEspace(new_espace, 0,        0,        1, half_y);
	  espace_d = createSubEspace(new_espace, 0,        middle_y, 1, half_y);
	  
	  morphing (espace_u, boxU(box));
	  deleteEspace(espace_u);
	  
	  morphing (espace_d, boxD(box));
	  deleteEspace(espace_d);
	}

  deleteEspace(new_espace);

/*
** For the test in test espace instead of recursive call
*/
/*
  printf("-------------------------------\n");
  printEspace(new_espace);
  printf("-------------------------------\n");
  printEspace(espace_ul);
  printf("-------------------------------\n");
  printEspace(espace_ur);
  printf("-------------------------------\n");
  printEspace(espace_dl);
  printf("-------------------------------\n");
  printEspace(espace_dr);
*/  
}
