#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "io.h"
#include "morphing.h"

#define FALSE 0
#define TRUE  1


char* ImageGetDefault = "Image/test.pnm";
char* ImageSetDefault = "result.pnm";
char* CurveDefault    = "Curve/test.esp";

void usage (char* name)
{
  printf ("\nUsage: %s [OPTION]\n\n"
		  "Projet d Image sur les courbe de bezier (maitrise)\n\n"
		  "  -h, --help\t cette aide\n"
		  "  -c, --curve\t change le fichier de definition des courbes (defaut: %s)\n"
		  "  -i, --input\t change le nom du fichier pnm lu (defaut: %s)\n"
		  "  -o, --output\t change le nom du fichier pnm cree (defaut: %s)\n"
		  "  -p, --print\t affiche les parametres\n"
		  "\n",
		  name,
		  CurveDefault,ImageGetDefault,ImageSetDefault
		  );
  	exit (0);
}

int main (int argc, char** argv)
{
  int next;
  int option_index;

  struct option long_options[] =
  {
	{ "help",    no_argument,       NULL, 'h' },
	{ "curve",   required_argument, NULL, 'c' },
    { "output",  required_argument, NULL, 'o' },
	{ "input",   required_argument, NULL, 'i' },
	{ "print",   no_argument,       NULL, 'p' },
    { 0, 0, 0, 0 }
  };

  char*  prog_name;

  bool flag_print = FALSE;

  espace_t* space;
  box_t     box;

  image_t* image_src;
  image_t* image_dst;

  char* image_get_name = NULL;
  char* image_set_name = NULL;
  char* curve_name     = NULL;

  prog_name    = argv [0];
  next         = 0;
  option_index = 0;

  /* parse options */

  while (TRUE)
    {
      next = getopt_long(argc, argv, "hc:o:i:p", long_options, &option_index);

	  if(next == -1) break;

	  switch (next)
		{
		case 'h':		
		  usage(prog_name);
		  break;
	  	
		case 'p':
		  flag_print = TRUE;
		  break;

		case 'i':
		  image_get_name = (char*) malloc (strlen(optarg)+1);
		  strcpy(image_get_name,optarg);
		  break;	  

		case 'o':
		  image_set_name = (char*) malloc (strlen(optarg)+1);
		  strcpy(image_set_name,optarg);		  
		  break;

		case 'c':
		  curve_name = (char*) malloc (strlen(optarg)+1);
		  strcpy(curve_name,optarg);
		  break;

		case '?':		
		default:
		  fprintf(stderr,"Essayez `%s --help' pour plus d'information.\n", 
				  prog_name);
		  exit (0);
		  break;
		}
    }

  if (image_get_name == NULL) image_get_name = ImageGetDefault;
  if (image_set_name == NULL) image_set_name = ImageSetDefault;
  if (curve_name     == NULL) curve_name     = CurveDefault;

  if(flag_print)
	{
	  printf("\n");
	  printf("Nom du fichier de definition de la courbe      : %s\n",curve_name);
	  printf("Nom du fichier correspondant a l'image lu      : %s\n",image_get_name);
	  printf("Nom du fichier correspondant a l'image a creer : %s\n",image_set_name);
	}

  space = readEspace(curve_name);

  if(!space)
	{
	  printf("Impossible de creer une courbe a partir du fichier donnee\n");
	  exit(0);
	}

  if(flag_print)
	{
	  printf("Nonbre de dimension des points de la courbe (2): %d\n",getDimension());
	  printf("Degree des courbes utilisees en X              : %d\n",getEspaceLine(space));
	  printf("Degree des courbes utilisees en Y              : %d\n",getEspaceColumn(space));
	  printf("\n");
	  printf("Affichage des points des courbes\n");
	  printEspace(space);
	}

  image_src = readPnm(image_get_name);

  if(!image_src)
	{
	  printf("Impossiblilite de creer une image a partir du nom de fichier donne\n");
	  exit(0);
	}

  image_dst = createImage(getImageSizeX(image_src),getImageSizeY(image_src));

  // Box init

  box.pos.x = 0;
  box.pos.y = 0;
  box.size.x  = getImageSizeX(image_src);
  box.size.y  = getImageSizeY(image_src);

  // Create new picture

  setMorphingSource (image_src);
  setMorphingResult (image_dst);
  morphing          (space,box);
  printf("\rProgression : done.        \n");
  // Save result

  writePnm(image_set_name,image_dst);

  // Free allocated (just a self check ..)

  deleteEspace(space);
  deleteImage(image_dst);

  if (curve_name     != CurveDefault)    free(curve_name);
  if (image_get_name != ImageGetDefault) free(image_get_name);
  if (image_set_name != ImageSetDefault) free(image_set_name);

  return 0;
}
